
{% extends "layout.html" %}
{% from "_helper.html" import image_button %}g
{% block content %}

<h3>VM Details</h3>

VM ID: {{id}}<br/>
Cloud Name: {{cloudname}}
<br/><br/>

<style>
table{
margin:0px
}
</style>

{{table_fun(server) | safe}}

{% endblock %}
