{% extends "layout.html" %}

{% block content %}

    <h2>List of pages</h2>
    <ul>
      <li><a href="/images">Images</a></li>
      <li><a href="/flavors">Flavors</a></li>
      <li><a href="/table/">Table</a></li>
      <li><a href="/">Home</a></li>
      <li><a href="/about">About</a></li>
      <li><a href="/contact">Contact</a></li>
      <li><a href="/profile">Profile</a></li>



    {% for page in pages %}
        <li>
            <a href="{{ url_for("page", path=page.path) }}">{{ page.title }}</a>
        </li>
    {% else %}
        <li>No stuff.</li>
    {% endfor %}
    </ul>

{% endblock content %}
