{% extends "layout.html" %}
{% from "_helper.html" import image_button %}g
{% block content %}

<h1> Projects </h1>

<form name="projects" action="/projects/" method="POST">
<table>
{% for projectname in projects %}


 <input type="radio" name="selected_project" value="{{ projectname }}" id="{{ projectname }}" {{ selected[projectname] }} onclick="document.forms['projects'].submit()" >
<b> {{ projectname }}</b> 



{% endfor %}
</table>
</form>
 
{% endblock %}
