{% extends "layout.html" %}
{% from "_helper.html" import image_button %}g
{% block content %}

Choose the default flavor </br>

<form name="flavor" action="" method="POST">
<table>
  <tr>
    <th> Default </th>
    <th> Id </th>
    <th> Name </th>
    <th> RAM </th>
    <th> Disk </th>
    <th> VCPUs </th>
    <th> Cost  </th>
  </tr>
  
  {% for name, cloud in clouds.iteritems() %}
  
  <tr> 
    <th colspan="6"> {{ name }} </th> 
  </tr>
  
  {% for id, flavor in cloud['flavors'].iteritems() %}
  
  <tr>
    <td> <input type="radio" name="{{ name }}" value="{{ flavor.name }}" id="{{ flavor.name }}"  {{ radioSelected[name][flavor.name] }}  onclick="document.forms['flavor'].submit()"> </td>
    <td> {{ flavor.id }} </td>
    <td> {{ flavor.name }} </td>
    <td> {{ flavor.ram }} </td>
    <td> {{ flavor.disk }} </td>
    <td> {{ flavor.vcpus }} </td>
    <td> TBD </td>
  </tr>
  
  {% endfor %}
  
  {% endfor %}
  
</table>

</form>
{% endblock %}
